#include <cstdio>
#include <stdlib.h> 

#include "math.h"

#include "glesclient.h"
#include "main.h"

bool GLESClient::loadShader(const char *shaderString, GLuint *shader, GLuint type){
	const GLchar *shaderStrings[] = {
		shaderString
	};

	*shader = GL_CHECK(glCreateShader(type));

	GL_CHECK(glShaderSource(*shader, 1, shaderStrings, NULL));
	GL_CHECK(glCompileShader(*shader));

	GLint status;

	GL_CHECK(glGetShaderiv(*shader, GL_COMPILE_STATUS, &status));

	if(status != GL_TRUE){
		GLint len;
		char *buffer = NULL;

		GL_CHECK(glGetShaderiv(*shader, GL_SHADER_SOURCE_LENGTH, &len));
		
		buffer = (char*) malloc(len);
		GL_CHECK(glGetShaderSource(*shader, len, NULL, buffer));
		printf("Shader debug source:\n%s\n", buffer);
		free(buffer);

		GL_CHECK(glGetShaderiv(*shader, GL_INFO_LOG_LENGTH, &len));
		
		buffer = (char*) malloc(len);
		GL_CHECK(glGetShaderInfoLog(*shader, len, NULL, buffer));
		printf("Shader error log:\n%s\n", buffer);
		free(buffer);

		return false;			
	}	
	return true;	
}

void GLESClient::init(){
	 /* Create and initialise an EGL display */
    m_display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    EGL_CHECK(eglInitialize(m_display, NULL, NULL));
    
    /* Get an EGL config */
    EGLint attribs[] = { EGL_NONE };
    EGLint numConfig = 0;
    
    EGL_CHECK(eglChooseConfig(m_display, attribs, &m_config, 1, &numConfig));
    
    /* Create the EGL window surface from this view */
    m_surface = EGL_CHECK(eglCreateWindowSurface(m_display, m_config, m_window, attribs));
    
    /* Create an EGL context */
    EGLint contextAttribs[] = {
        EGL_CONTEXT_CLIENT_VERSION, 2,
        EGL_NONE
    };
    


    m_context = EGL_CHECK(eglCreateContext(m_display, m_config, NULL, contextAttribs));
    
    /* Make the context current */
    EGL_CHECK(eglMakeCurrent(m_display, m_surface, m_surface, m_context));

		if(!loadShader(m_vertexShaderSrc.c_str(), &m_vertexShader, GL_VERTEX_SHADER)){
			printf("Could not load vertex shader\n");
			return;
		}
		printf("Vertex shader loaded\n");

		if(!loadShader(m_fragmentShaderSrc.c_str(), &m_fragmentShader, GL_FRAGMENT_SHADER)){
			printf("Could not load fragment shader\n");
			return;
		}
		printf("Fragment shader loaded\n");
 
    /* Create the program and attach the shaders */
    m_program = GL_CHECK(glCreateProgram());
    
    GL_CHECK(glAttachShader(m_program, m_vertexShader));
    GL_CHECK(glAttachShader(m_program, m_fragmentShader));
		    
    /* Link the program */
    GL_CHECK(glLinkProgram(m_program));
	
		GLint result = GL_FALSE;
		GL_CHECK(glGetProgramiv(m_program, GL_LINK_STATUS, &result));
		if(result != GL_TRUE){
			GLint logLength;
			GL_CHECK(glGetProgramiv(m_program, GL_INFO_LOG_LENGTH, &logLength));
			char infoLog[logLength];
			glGetProgramInfoLog(m_program, logLength, NULL, infoLog);
			printf("Error %s\n", infoLog);
			return;
		}


    /* Select the program */
    GL_CHECK(glUseProgram(m_program));
    
    /* Turn on backface culling and depth testing */
		GL_CHECK(glEnable(GL_BLEND));
    GL_CHECK(glEnable(GL_CULL_FACE));
    GL_CHECK(glEnable(GL_DEPTH_TEST));


		m_eglInitialized = true;

		init_resources();


}
GLuint uniform_fade;
GLuint uniform_m_transform;

GLuint vbo_cube_verticies, vbo_cube_colors, ibo_cube_elements;

void GLESClient::init_resources(){
	GLfloat cube_verticies[] = {
		//front
		-1.0f, -1.0f, 1.0f,
		1.0f, -1.0f, 1.0f,
		1.0f, 1.0f, 1.0f,
		-1.0f, 1.0f, 1.0f,
		//back
		-1.0f, -1.0f, -1.0f,
		1.0f, -1.0f, -1.0f,
		1.0f, 1.0f, -1.0f,
		-1.0f, 1.0f, -1.0f,
	};

	GLfloat cube_colors[] = {
		//front
		1.0f, 0.0f, 0.0f,
		0.0f, 1.0f, 0.0f,
		0.0f, 0.0f, 1.0f,
		1.0f, 1.0f, 1.0f,
		//back
		1.0f, 0.0f, 0.0f,
		0.0f, 1.0f, 0.0f,
		0.0f, 0.0f, 1.0f,
		1.0f, 1.0f, 1.0f,
	};

	GLushort cube_elements[] = {
		//front
		0,1,2,
		2,3,0,
		//top
		3,2,6,
		6,7,3,
		//back
		7,6,5,
		5,4,7,
		//bottom
		4,5,1,
		1,0,4,
		//left
		4,0,3,
		3,7,4,
		//right
		1,5,6,
		6,2,1
	};

	GL_CHECK(glGenBuffers(1, &vbo_cube_verticies));
	GL_CHECK(glBindBuffer(GL_ARRAY_BUFFER, vbo_cube_verticies));
	GL_CHECK(glBufferData(GL_ARRAY_BUFFER, sizeof(cube_verticies), cube_verticies, GL_STATIC_DRAW));

	GL_CHECK(glGenBuffers(1, &vbo_cube_colors));
	GL_CHECK(glBindBuffer(GL_ARRAY_BUFFER, vbo_cube_colors));
	GL_CHECK(glBufferData(GL_ARRAY_BUFFER, sizeof(cube_colors), cube_colors, GL_STATIC_DRAW));

	GL_CHECK(glGenBuffers(1, &ibo_cube_elements));
	GL_CHECK(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo_cube_elements));
	GL_CHECK(glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(cube_elements), cube_elements, GL_STATIC_DRAW));
	GL_CHECK(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0));
	
	positionSlot = GL_CHECK(glGetAttribLocation(m_program, "v_position"));
	colorSlot = EGL_CHECK(glGetAttribLocation(m_program, "v_color"));
}

void GLESClient::cycle(){
	if(!m_eglInitialized){ return; }
}

void GLESClient::draw(){
	if(!m_eglInitialized){ return; }

	
	GL_CHECK(glEnableVertexAttribArray(positionSlot));
	GL_CHECK(glEnableVertexAttribArray(colorSlot));

	GL_CHECK(glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA));

	//Clear background with a grey
	GL_CHECK(glClearColor(0.2, 0.2, 0.2, 1.0));
	GL_CHECK(glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT));

	GL_CHECK(glBindBuffer(GL_ARRAY_BUFFER, vbo_cube_verticies));
	GL_CHECK(glVertexAttribPointer( positionSlot, 3, GL_FLOAT, GL_FALSE, 0, 0));

	GL_CHECK(glBindBuffer(GL_ARRAY_BUFFER, vbo_cube_colors));
	GL_CHECK(glVertexAttribPointer( colorSlot, 3, GL_FLOAT, GL_FALSE, 0, 0));
	
	int size;
	GL_CHECK(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo_cube_elements));
	GL_CHECK(glGetBufferParameteriv(GL_ELEMENT_ARRAY_BUFFER, GL_BUFFER_SIZE, &size));
	GL_CHECK(glDrawElements(GL_TRIANGLES, size/sizeof(GLushort), GL_UNSIGNED_SHORT, 0));	

	EGL_CHECK(eglSwapBuffers(m_display, m_surface));	
	
	GL_CHECK(glDisableVertexAttribArray(positionSlot));
	GL_CHECK(glDisableVertexAttribArray(colorSlot));
}
